#    Diagnostic and test task for the AGATA Digitizer interface

set frame  Agata
set title "Agata Digitizer Access Client"
set icon   Agata

set started 0

set LogFlag 1

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set AgataServerList {localhost}
set AgataServer [lindex $AgataServerList 0]

set AgataPortList {8015}
set AgataPort [lindex $AgataPortList 0]


set FpgaList { \
       "Core: Virtex for Segment ADC card 1" \
       "Core: Virtex for Segment ADC card 2" \
       "Core: Virtex for core ADCs" \
       "Core: Main Board" \
       "Segment: Virtex for Segment ADC card 1" \
       "Segment: Virtex for Segment ADC card 2" \
       "Segment: Virtex for Segment ADC card 3" \
       "Segment: Virtex for Segment ADC card 4" \
       "Segment: Main Board" \
}
set FpgaIDs {0 1 2 3 8 9 10 11 12}
set Fpga "Core: Main Board"

set AddressModifierList {A16 A32}
set AddressModifier [lindex $AddressModifierList 0]

set NumDigitizer 0
set DigitizerList {}
set Digitizer -1

set Address 0x0
set Data 0x0

set verbose 0


proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode]} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}



proc make-frame {} {
   global env AgataServerList AgataPortList AgataServer AgataPort
   global DigitizerList FpgaList AddressModifierList Address Data

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0
    :text XPORTADDRT         :label "Agata Digitizer Web Service" :w 20 :value $AgataServer
    :choice-stack XPORTADDRC :label "" :strings $AgataServerList 
    :hskip 50
    :text XPORTPORTT         :label "TCP Port" :w 12 :value $AgataPort
    :choice-stack XPORTPORTC :label "" :strings $AgataPortList 

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0
     :button CLEAR   :label "Clear log window" :command ":modify LOG :empty T"
     :hskip 100; :col-gap 20
     :checkbox VERBOSE :label "Enable Verbose Option" 

  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P3 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :choice-stack DIGITIZER :label "Digitizer"  :exclusive T :menu $DigitizerList :value 0
      :choice-stack FPGA      :label "FPGA"  :exclusive T :strings $FpgaList :value 3
      :hskip 120
      :choice-stack ADDRMOD   :label "Address Modifier"  :exclusive T :strings $AddressModifierList :value 0
      :next-row T
      :text ADDR              :label "Address" :w 10 :value $Address
      :text DATA     :label "Data"    :w 10 :value $Data
      :button GET    :label "Do Get"  :w 8
      :button SET    :label "Do Set"  :w 8

   open-frame

  :galley_size LOG $rows $cols

#    focus [:path CMD].entry

   :show T
}


proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}

proc start args {
   global started

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
}

proc click arg {
   global history verbose longbuffer datafile usedatafile
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch -glob $w  {

      XPORTADDRT    {click-server $v}
      XPORTADDRC    {click-server $i}
      XPORTPORTT    {click-port $v}
      XPORTPORTC    {click-port $i}
      
      VERBOSE       {set verbose $v}

      FPGA          {click-fpga $i}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click-server {v} {
    global AGATADigitizerAccessServerHost AGATADigitizerAccessServerPort
    global AgataServerList AgataServer AgataPort
    global NumDigitizer DigitizerList Digitizer
    global verbose

    set  AgataServer $v
    :modify XPORTADDRT :value $AgataServer
    :modify XPORTADDRC :value [lsearch $AgataServerList $AgataServer]

    set AGATADigitizerAccessServerHost $AgataServer
    set AGATADigitizerAccessServerPort $AgataPort
    package require AGATADigitizerAccessClient

    loadsource agata   ;#  until I have a better way to configure the Server Access

    set NumDigitizer [AGATADigitizerAccessClient__Init]

    insert "connected to Agata Digitizer Web Service at host $AgataServer port $AgataPort"
    insert "discovered $NumDigitizer digitizer(s)"

    set DigitizerList {}
    for {set i 0} {$i < $NumDigitizer} {incr i} {lappend DigitizerList $i}
    :modify DIGITIZER :menu $DigitizerList :value 0
    click-digitizer 0

}

proc click-port {v} {
    global AgataPortList AgataPort

    set  AgataPort $v
    :modify XPORTPORTT :value $AgataPort
    :modify XPORTPORTC :value [lsearch $AgataPortList $AgataPort]

}

proc click-data {v} {
    global Data

    set Data $v

    insert "Data= [format "0x%x" $Data]"
}

proc click-addr {v} {
    global Address

    set Address $v

    insert "Address= [format "0x%x" $Address]"
}

proc click-digitizer {v} {
    global Digitizer

    set Digitizer $v

    insert "Digitizer= $Digitizer"
}

proc click-fpga {v} {
    global Fpga

    set Fpga $v

    insert "Fpga= $Fpga"
}

proc click-addrmod {v} {
    global AddressModifierList AddressModifier

    set AddressModifier [lindex $AddressModifierList $v]

    insert "AddressModifier= $AddressModifier"
}

proc click-set {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier Data
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $Address $AddressModifier $Data} m]
    if {$rc != 0} {
        insert "Do Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Set OK"
    }
}

proc click-get {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $Address $AddressModifier} m]
    if {$rc != 0} {
        insert "Do Get returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Get returned $m ([format "0x%x" $m])"
    }
}

